#include "informix-cplus-2.53.UC2.hh"


/*
*  UESQLC::informix-cplus-2.53.UC2 internal variables initiation.
*/

TInformixManager   htInformixManager;


/*
*  Name:      test-base-1.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 1999/04/20 14:04:18 $
*  Revision:  $Revision: 1.1 $
*
*  Copyright (C) 1996-1998  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/string.hh>
#include <mpcl/config_processor.hh>

using namespace mpcl;


string GetErrorMessage (void)
{

  string   yMessage;

  if ( SQLSTATE == "00000" )
  {
    yMessage = "successful completion";
  }
  else if ( SQLSTATE == "02000" )
  {
    yMessage = "no data";
  }
  else if ( SQLSTATE == "08001" )
  {
    yMessage = "connection exception: SQL-client unable to establish SQL-connection";
  }
  else if ( SQLSTATE == "08002" )
  {
    yMessage = "connection exception: connection name in use";
  }
  else if ( SQLSTATE == "08003" )
  {
    yMessage = "connection exception: connection does not exist";
  }
  else if ( SQLSTATE == "08007" )
  {
    yMessage = "connection exception: transaction resolution unknown";
  }
  else if ( SQLSTATE == "21000" )
  {
    yMessage = "cardinality violation";
  }
  else if ( SQLSTATE == "34000" )
  {
    yMessage = "invalid cursor name";
  }
  else if ( SQLSTATE == "40003" )
  {
    yMessage = "transaction rollback: statement completion unknown";
  }
  else if ( SQLSTATE == "42000" )
  {
    yMessage = "syntax error or access rule violation";
  }
  else
  {
    yMessage = "unknown error";
  }
  return yMessage;

}  /* GetErrorMessage() */


/*
*  Parameters and arguments:
*
*    {--hostname, -o}=HOSTNAME
*    {--username, -u}=USERNAME
*    {--password, -p}=PASSWORD
*/
int main (int argc, const char* argv[])
{

  TConfigProcessor   tConfig;
  size_t             zRecordCount = 0;

  TEST_INIT ("test-base-1");
  TEST_START_WATCH;

  string   yVar0;
  string   yVar1;
  string   yVar2;
  string   yAttribName;
  string   yCursorName;
  string   yUserName;
  string   yHostName;
  string   yCodeValue = "'002'";

  /*
  *  Define string constants names for options.
  */ 
  tConfig.addOption ("hostname", "o", "localhost");
  tConfig.addOption ("username", "u", getenv ("USER"));
  tConfig.addOption ("password", "p", "");
  tConfig.setCommandLineParameters (argc, argv);
  tConfig.processOptions();
  
  yHostName = Format ("'%s'", tConfig ["hostname"].value().c_str());
  if ( tConfig ["password"].value().empty() )
  {
    yUserName = Format ("'%s'", tConfig ["username"].value().c_str());
  }
  else
  {
    yUserName = Format ( "'%s/%s'"                            ,
                         tConfig ["username"].value().c_str() ,
                         tConfig ["password"].value().c_str() );
  }

  TEST_WRITE_INFO ("HOSTNAME = %s", yHostName.c_str());
  TEST_WRITE_INFO ("USER     = %s", yUserName.c_str());
  if ( !yHostName.empty() && !yUserName.empty() )
  {
{
/*
*  SQL instructions.
*/


{
mpcl::xstring   hstSqlServerDescriptor;
mpcl::xstring   hstSqlServerName;
mpcl::xstring   hstDatabaseName;
mpcl::xstring   hstConnectionName;
const char*     hpakcVariables[] = { NULL , yHostName.c_str(), yUserName.c_str() };

/*
*  '%s' has been omitted.
*/
hstSqlServerDescriptor = IX_INSTANTIATE ("%s",  0, hpakcVariables);
hstConnectionName      = "DEFAULT";
hstSqlServerDescriptor.suppress ('\'');
hstSqlServerName                 = hstSqlServerDescriptor.substr (0, hstSqlServerDescriptor.find ('/'));
hstDatabaseName                  = hstSqlServerDescriptor.substr (hstSqlServerDescriptor.find ('/') + 1);
htInformixManager.pcHost         = (char*) hstSqlServerName.c_str();
htInformixManager.pcDatabaseName = (char*) hstDatabaseName.c_str();
if ( !htInformixManager.pcDatabaseName )
{
  htInformixManager.pcDatabaseName = getenv ("USER");
}
else if ( !strlen (htInformixManager.pcDatabaseName) )
{
  htInformixManager.pcDatabaseName = getenv ("USER");
}
htInformixManager.ptItconnection = PQsetdb (htInformixManager.pcHost, NULL, NULL, NULL, htInformixManager.pcDatabaseName);
if ( PQstatus (htInformixManager.ptItconnection) != CONNECTION_BAD )
{
  SQLCODE  = 0;
  SQLSTATE = "00000";
}
else
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  SQL-client unable to establish SQL-connection
  */
  SQLCODE  = -1;
  SQLSTATE = "08001";
  goto error;
}
if ( htInformixManager.isConnected (hstConnectionName) )
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  connection name in use
  */
  SQLCODE  = -1;
  SQLSTATE = "08002";
  goto error;
}
htInformixManager.connect (hstConnectionName, htInformixManager.ptItconnection);
htInformixManager.setCurrentConnection (hstConnectionName);
}
htInformixManager.stSqlInstruction = Format ("CREATE TABLE JOBS (CODE CHAR (3) NOT NULL, WORKER_COUNT INT4);");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("CREATE TABLE WORKERS (WORK_CODE CHAR (3) NOT NULL, NAME CHAR (40) NOT NULL, HOURS DECIMAL (5));");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("ALTER TABLE WORKERS ADD COLUMN STATE CHAR (40);");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("ALTER TABLE WORKERS ADD COLUMN PHONE_NUMBER INT4;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("DELETE FROM JOBS WHERE ( CODE = %s );", yCodeValue.c_str());
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("DELETE FROM JOBS WHERE ( CODE = '001' );");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("DELETE FROM WORKERS;");
IXS_EXEC ("DELETE FROM WORKERS;", htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO JOBS VALUES ('001', 100);");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO JOBS VALUES ('002', 250);");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO WORKERS VALUES ('001', 'myname', 40, 'Andalucia', 5550000);");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO WORKERS VALUES ('001', 'yourname', 35, 'Extremadura', 5550001);");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO WORKERS VALUES ('001', 'hisname', 25, 'Catalunya', 5550002);");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO WORKERS VALUES ('002', 'john', 35, 'New England', 5550003);");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO WORKERS VALUES ('002', 'james', 35, 'Washington', 5550004);");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS WHERE ( NAME = 'myname' );");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS;");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);

{
int   J = 0;

htInformixManager.stSqlInstruction = Format ("DECLARE AUX_CURSOR CURSOR FOR SELECT HOURS FROM WORKERS, JOBS WHERE ( WORK_CODE = CODE );");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto cardinality_check;);
IX_EXEC ("FETCH ALL IN AUX_CURSOR;", htInformixManager.ptItconnection, goto cardinality_check;);
if ( IX_NOT_FOUND_CONDITION )
{
  /* 
  *  Condition:  no data
  */ 
  SQLCODE = 101; SQLSTATE = "02000";
  if ( htInformixManager.ptPgresult ) PQclear (htInformixManager.ptPgresult);
  IXH_EXEC ("CLOSE AUX_CURSOR;", htInformixManager.ptItconnection);
  goto not_found;
}
else if ( PQntuples (htInformixManager.ptPgresult) != 1 )
{
  /* 
  *  Condition:  cardinality violation
  */ 
  SQLCODE = -1; SQLSTATE = "21000"; 
  if ( htInformixManager.ptPgresult ) PQclear (htInformixManager.ptPgresult);
  IXH_EXEC ("CLOSE AUX_CURSOR;", htInformixManager.ptItconnection);
  goto cardinality_check;
}
else
{
  SQLCODE = 0; SQLSTATE = "00000";
  yVar1 = PQgetvalue (htInformixManager.ptPgresult, 0, J++);
  if ( htInformixManager.ptPgresult ) PQclear (htInformixManager.ptPgresult);
  IXS_EXEC ("CLOSE AUX_CURSOR;", htInformixManager.ptItconnection, goto cardinality_check;);
}
}

}
    TEST_FAIL;


  cardinality_check:

    TEST_STRINGS (SQLSTATE.c_str(), "21000");
    yAttribName = "WORK_CODE";
    yCursorName = "MYCURSOR";
{
/*
*  SQL instructions.
*/

{
int   J = 0;

htInformixManager.stSqlInstruction = Format ("DECLARE AUX_CURSOR CURSOR FOR SELECT COUNT (*) FROM WORKERS WHERE ( NAME = 'myname' );");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
IX_EXEC ("FETCH ALL IN AUX_CURSOR;", htInformixManager.ptItconnection, goto error;);
if ( IX_NOT_FOUND_CONDITION )
{
  /* 
  *  Condition:  no data
  */ 
  SQLCODE = 101; SQLSTATE = "02000";
  if ( htInformixManager.ptPgresult ) PQclear (htInformixManager.ptPgresult);
  IXH_EXEC ("CLOSE AUX_CURSOR;", htInformixManager.ptItconnection);
  goto not_found;
}
else if ( PQntuples (htInformixManager.ptPgresult) != 1 )
{
  /* 
  *  Condition:  cardinality violation
  */ 
  SQLCODE = -1; SQLSTATE = "21000"; 
  if ( htInformixManager.ptPgresult ) PQclear (htInformixManager.ptPgresult);
  IXH_EXEC ("CLOSE AUX_CURSOR;", htInformixManager.ptItconnection);
  goto error;
}
else
{
  SQLCODE = 0; SQLSTATE = "00000";
  yVar2 = PQgetvalue (htInformixManager.ptPgresult, 0, J++);
  if ( htInformixManager.ptPgresult ) PQclear (htInformixManager.ptPgresult);
  IXS_EXEC ("CLOSE AUX_CURSOR;", htInformixManager.ptItconnection, goto error;);
}
}
htInformixManager.stSqlInstruction = Format ("UPDATE WORKERS SET PHONE_NUMBER = WORKERS.PHONE_NUMBER WHERE ( ( WORKERS.NAME = 'myname' ) AND ( WORKERS.NAME = NAME ) );");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
/*
*  'CASCADE' has been omitted.
*/
htInformixManager.stSqlInstruction = Format ("REVOKE ALL PRIVILEGES ON JOBS FROM PUBLIC;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
/*
*  'RESTRICT' has been omitted.
*/
htInformixManager.stSqlInstruction = Format ("REVOKE ALL PRIVILEGES ON WORKERS FROM PUBLIC;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("GRANT SELECT ON JOBS TO PUBLIC;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("CREATE VIEW VIEW1 AS SELECT * FROM JOBS;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
/*
*  'CASCADE' has been omitted.
*/
htInformixManager.stSqlInstruction = Format ("DROP VIEW VIEW1;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
htInformixManager.stSqlInstruction = Format ("DECLARE MYCURSOR CURSOR FOR SELECT CODE FROM JOBS WHERE ( CODE = '001' );");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);

{
int   J = 0;

htInformixManager.stSqlInstruction = Format ("FETCH IN %s;", yCursorName.c_str());
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
if ( IX_NOT_FOUND_CONDITION )
{
  /* 
  *  Condition:  no data
  */ 
  SQLCODE = 101; SQLSTATE = "02000";
  if ( htInformixManager.ptPgresult ) { PQclear (htInformixManager.ptPgresult); }
  goto not_found;
}
else
{
  yVar0 = PQgetvalue (htInformixManager.ptPgresult, 0, J++);
  if ( htInformixManager.ptPgresult ) { PQclear (htInformixManager.ptPgresult); }
}
}
htInformixManager.stSqlInstruction = Format ("CLOSE MYCURSOR;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);

/*
*  'CASCADE' has been omitted.
*/
htInformixManager.stSqlInstruction = Format ("DROP TABLE JOBS;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);

}

    TEST_STRINGS (SQLSTATE.c_str(), "00000");
    TEST_STRINGS (yVar2.c_str(),   "4");
    TEST_STRINGS (yVar0.c_str(),   "001");

{
/*
*  SQL instructions.
*/
htInformixManager.stSqlInstruction = Format ("DECLARE MYCURSOR CURSOR FOR SELECT %s, NAME, HOURS FROM WORKERS;", yAttribName.c_str());
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);



}
    
    if ( _ptTargetFile )
    {
      fprintf (_ptTargetFile, "%s\n", "WORK_CODE NAME                                     HOURS");
      fprintf (_ptTargetFile, "%s\n", "========= ======================================== =====");
    }
    for (;;)
    {
{
/*
*  SQL instructions.
*/
{
int   J = 0;

htInformixManager.stSqlInstruction = Format ("FETCH IN MYCURSOR;");
IX_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);
if ( IX_NOT_FOUND_CONDITION )
{
  /* 
  *  Condition:  no data
  */ 
  SQLCODE = 101; SQLSTATE = "02000";
  if ( htInformixManager.ptPgresult ) { PQclear (htInformixManager.ptPgresult); }
  goto loop_end;
}
else
{
  yVar0 = PQgetvalue (htInformixManager.ptPgresult, 0, J++);
yVar1 = PQgetvalue (htInformixManager.ptPgresult, 0, J++);
yVar2 = PQgetvalue (htInformixManager.ptPgresult, 0, J++);
  if ( htInformixManager.ptPgresult ) { PQclear (htInformixManager.ptPgresult); }
}
}

}
      if ( _ptTargetFile )
      {
        fprintf (_ptTargetFile, "%9s %40s %5s\n", yVar0.c_str(), yVar1.c_str(), yVar2.c_str());
      }
      ++zRecordCount;
    }


  loop_end:

{
/*
*  SQL instructions.
*/
htInformixManager.stSqlInstruction = Format ("CLOSE MYCURSOR;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);

{ if ( htInformixManager.existsDefaultConnection() )
{
  SQLCODE  = 0;
  SQLSTATE = "00000";
  htInformixManager.disconnectCurrent();
}
else
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  connection does not exist
  */
  SQLCODE  = -1;
  SQLSTATE = "08003";
  goto error;
}
 }

}

{
/*
*  SQL instructions.
*/
{
mpcl::xstring   hstSqlServerDescriptor;
mpcl::xstring   hstSqlServerName;
mpcl::xstring   hstDatabaseName;
mpcl::xstring   hstConnectionName;
const char*     hpakcVariables[] = { NULL , yHostName.c_str(), yUserName.c_str() };

/*
*  '%s' has been omitted.
*/
hstSqlServerDescriptor = IX_INSTANTIATE ("%s",  0, hpakcVariables);
hstConnectionName      = IX_INSTANTIATE ("_'myconnection'", 1, hpakcVariables);
hstSqlServerDescriptor.suppress ('\'');
hstConnectionName.suppress ('\'');
hstSqlServerName                 = hstSqlServerDescriptor.substr (0, hstSqlServerDescriptor.find ('/'));
hstDatabaseName                  = hstSqlServerDescriptor.substr (hstSqlServerDescriptor.find ('/') + 1);
htInformixManager.pcHost         = (char*) hstSqlServerName.c_str();
htInformixManager.pcDatabaseName = (char*) hstDatabaseName.c_str();
if ( !htInformixManager.pcDatabaseName )
{
  htInformixManager.pcDatabaseName = getenv ("USER");
}
else if ( !strlen (htInformixManager.pcDatabaseName) )
{
  htInformixManager.pcDatabaseName = getenv ("USER");
}
htInformixManager.ptItconnection = PQsetdb (htInformixManager.pcHost, NULL, NULL, NULL, htInformixManager.pcDatabaseName);
if ( PQstatus (htInformixManager.ptItconnection) != CONNECTION_BAD )
{
  SQLCODE  = 0;
  SQLSTATE = "00000";
}
else
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  SQL-client unable to establish SQL-connection
  */
  SQLCODE  = -1;
  SQLSTATE = "08001";
  goto error;
}
if ( htInformixManager.isConnected (hstConnectionName) )
{
  /*
  *  Condition:           connection exception
  *  Class subcondition:  connection name in use
  */
  SQLCODE  = -1;
  SQLSTATE = "08002";
  goto error;
}
htInformixManager.connect (hstConnectionName, htInformixManager.ptItconnection);
htInformixManager.setCurrentConnection (hstConnectionName);
}
/*
*  'CASCADE' has been omitted.
*/
htInformixManager.stSqlInstruction = Format ("DROP TABLE WORKERS;");
IXS_EXEC (htInformixManager.stSqlInstruction.c_str(), htInformixManager.ptItconnection, goto error;);

}
    goto end;

    
  error:

    cerr << "SQLSTATE="    << SQLSTATE << endl;
    cerr << "SQLCODE="     << SQLCODE  << endl;
    cerr << "DESCRIPTION=" << GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (SQLCODE);  
    goto end;

    
  not_found:

    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH;
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  /* main() */

