#include "oracle-oci-7.x.hh"


/*
*  UESQLC::oracle-oci-7.x template explicit instantiations.
*/

template class TCollection<Lda_Def*>;
template class TBaseContainer<Lda_Def*>;
template class TLinkedContainer<Lda_Def*>;
template class TBaseMap<string, Lda_Def*>;
template class TLinkedMap<string, Lda_Def*>;

template class TCollection<Cda_Def*>;
template class TBaseContainer<Cda_Def*>;
template class TLinkedContainer<Cda_Def*>;
template class TBaseMap<string, Cda_Def*>;
template class TLinkedMap<string, Cda_Def*>;

template class TCollection<ub1*>;
template class TBaseContainer<ub1*>;
template class TLinkedContainer<ub1*>;
template class TBaseMap<string, ub1*>;
template class TLinkedMap<string, ub1*>;

template class TCollection<size_t>;
template class TBaseContainer<size_t>;
template class TLinkedContainer<size_t>;
template class TBaseQueue<string>;
template class TLinkedQueue<string>;


/*
*  C O N S T R U C T O R S
*/

TOci7xManager::~TOci7xManager (void)
{

  finish();

}  /* ~TOci7xManager() */


int TOci7xManager::initiate (void)
{

  int   iCode = 0;

  if ( !gIsThereConnection )
  {
    iCode = connect (kyDefaultConnectionName, (text*) "/");
  }
  return iCode;

}  /* initiate() */


void TOci7xManager::finish (void)
{

  for (register size_t J = 0, zMapCardinal = tCda_defMap.linksCardinal(); ( J < zMapCardinal ) ;J++)
  {
    closeCursor (tCda_defMap.key (J));
  }
  disconnectAll();

}  /* finish() */


int TOci7xManager::connect (const string& rkyCONNECTION_NAME, text* pktUSER_ID)
{

  int        iCode        = -1;
  Lda_Def*   psNewLda_def = new Lda_Def;
  ub1*       pcNewHda     = new ub1 [HDA_SIZE];

  if ( psNewLda_def && pcNewHda )
  {
    memset (pcNewHda, 0, HDA_SIZE);
    iCode = olog ( psNewLda_def     ,
		   pcNewHda         ,
		   pktUSER_ID       ,
		   (sword) -1       ,
		   (text*) 0        ,
		   (sword) -1       ,
		   (text*) 0        ,
		   (sword) -1       ,
		   (ub4) OCI_LM_DEF );
  }

  if ( !iCode )
  {
    tLda_defMap.bind (rkyCONNECTION_NAME, psNewLda_def);
    tHdaMap.bind (rkyCONNECTION_NAME, pcNewHda);
    stCurrentConnection = rkyCONNECTION_NAME;
    gIsThereConnection  = true;
  }
  else
  {
    /*
    *  Condition:           connection exception
    *  Class subcondition:  SQL-client unable to establish SQL-connection
    */
    gBypassOracleCode = true;
    SQLCODE           = -1;
    SQLSTATE          = "08001";
    if ( psNewLda_def )
    {
      delete psNewLda_def;
    }
    if ( pcNewHda )
    {
      delete[] pcNewHda;
    }
  }                                                                                               
  return iCode;

}  /* connect() */


int TOci7xManager::disconnect (const string& rkyCONNECTION_NAME)
{

  int   iCode = -1;

  if ( !tLda_defMap.isBinded (rkyCONNECTION_NAME) )
  {
    /*
    *  Condition:           connection exception
    *  Class subcondition:  connection does not exist
    */
    gBypassOracleCode = true;
    SQLCODE           = -1;
    SQLSTATE          = "08003";
  }
  else
  {
    iCode = ologof (tLda_defMap [rkyCONNECTION_NAME]);
    delete   tLda_defMap [rkyCONNECTION_NAME];
    delete[] tHdaMap     [rkyCONNECTION_NAME];
    tLda_defMap.unbind (rkyCONNECTION_NAME);
    tHdaMap.unbind (rkyCONNECTION_NAME);
    if ( rkyCONNECTION_NAME == stCurrentConnection )
    {
      stCurrentConnection.erase();
      gIsThereConnection = false;
    }
  }
  return iCode;

}  /* disconnect() */


int TOci7xManager::disconnectCurrent (void)
{

  return disconnect (stCurrentConnection);

}  /* disconnectCurrent() */


int TOci7xManager::disconnectAll (void)
{

  register size_t   J;
  register size_t   zMapCardinal;
  int               iCode = 0;

  for (J = 0, zMapCardinal = tLda_defMap.linksCardinal(); ( ( J < zMapCardinal ) && !iCode ) ;J++)
  {
    iCode = disconnect (tLda_defMap.key (J));
  }
  return iCode;

}  /* disconnectAll() */


Lda_Def* TOci7xManager::setCurrentConnection (const string& rkyCONNECTION_NAME)
{

  Lda_Def*   ptCurrentLda_def = NULL;

  if ( !tLda_defMap.isBinded (rkyCONNECTION_NAME) )
  {
    /*
    *  Condition:           connection exception
    *  Class subcondition:  connection does not exist
    */
    gBypassOracleCode = true;
    SQLCODE           = -1;
    SQLSTATE          = "08003";
  }
  else
  {
    stCurrentConnection = rkyCONNECTION_NAME;
    ptCurrentLda_def    = tLda_defMap [rkyCONNECTION_NAME];
  }
  return ptCurrentLda_def;

}  /* setCurrentConnection() */


int TOci7xManager::openCursor (const string& rkyCURSOR_NAME)
{

  int        iCode            = -1;
  Lda_Def*   ptCurrentLda_def = currentConnection();
  Cda_Def*   ptNewCda_def     = new Cda_Def;

  if ( ptCurrentLda_def && ptNewCda_def )
  {
    iCode = oopen (ptNewCda_def, ptCurrentLda_def, (text*) 0, -1, -1, (text*) 0, -1);
    if ( !iCode )
    {
      tCda_defMap.bind (rkyCURSOR_NAME, ptNewCda_def);
    }
    else
    {
      delete ptNewCda_def;
    }
  }
  return iCode;

}  /* openCursor() */


int TOci7xManager::closeCursor (const string& rkyCURSOR_NAME)
{

  int iCode = -1;

  if ( tCda_defMap.isBinded (rkyCURSOR_NAME) )
  {
    iCode = oclose (tCda_defMap [rkyCURSOR_NAME]);
    tCda_defMap.unbind (rkyCURSOR_NAME);
  }
  else
  {
    /*
    *  Condition:           invalid cursor name
    *  Class subcondition:  (no subclass)
    */
    gBypassOracleCode = true;
    SQLCODE           = -1;
    SQLSTATE          = "34000";
  }
  return iCode;

}  /* closeCursor() */


void TOci7xManager::initiateEmbeddedVariableMap (const char* pkcNULL_INITIATOR...)
{

  const char*   pkcVariableValue;
  va_list       tVa_list;

  tEmbeddedVariableValueQueue.erase();
  va_start (tVa_list, pkcNULL_INITIATOR);
  while ( NULL != (pkcVariableValue = va_arg (tVa_list, const char*)) )
  {
    tEmbeddedVariableValueQueue.put (pkcVariableValue);
  }
  va_end (tVa_list);

}  /* initiateEmbeddedVariableMap() */


string TOci7xManager::instantiateString (const char* pkcFORMAT)
{

  size_t    zOffset;
  size_t    zStart = 0;
  string   stInstantiation (pkcFORMAT);
 
  while ( stInstantiation.findFirst ("%s", zOffset, zStart) )
  {
    zStart = zOffset + tEmbeddedVariableValueQueue.head().length();
    stInstantiation.replace ("%s", tEmbeddedVariableValueQueue.get().c_str(), zOffset);
  }
  return stInstantiation;

}  /* instantiateString() */


/*
*  S E L E C T O R S
*/

bool TOci7xManager::existsDefaultConnection (void) const
{

  return !stCurrentConnection.isEmpty();

}  /* existsDefaultConnection() */


bool TOci7xManager::isConnected (const string& rkyCONNECTION_NAME) const
{

  return tLda_defMap.isBinded (rkyCONNECTION_NAME);

}  /* isConnected() */


Lda_Def* TOci7xManager::currentConnection (void)
{

  Lda_Def*   psLda_def = NULL;

  if ( !initiate() )
  {
    psLda_def = tLda_defMap [stCurrentConnection];
  }
  return psLda_def;

}  /* currentConnection() */


Lda_Def* TOci7xManager::connection (const string& rkyCONNECTION_NAME)
{

  Lda_Def*   psLda_def = NULL;

  if ( !initiate() )
  {
    if ( tLda_defMap.isBinded (rkyCONNECTION_NAME) )
    {
      psLda_def = tLda_defMap [rkyCONNECTION_NAME];
    }
    else
    {
      /*
      *  Condition:           connection exception
      *  Class subcondition:  connection does not exist
      */
      gBypassOracleCode = true;
      SQLCODE           = -1;
      SQLSTATE          = "08003";
    }
  }
  return psLda_def;

}  /* connection() */


Cda_Def* TOci7xManager::cursor (const string& rkyCURSOR_NAME) const
{

  Cda_Def*   psCda_def = NULL;

  if ( tCda_defMap.isBinded (rkyCURSOR_NAME) )
  {
    psCda_def = tCda_defMap [rkyCURSOR_NAME];
  }
  else
  {
    /*
    *  Condition:           invalid cursor name
    *  Class subcondition:  (no subclass)
    */
    gBypassOracleCode = true;
    SQLCODE           = -1;
    SQLSTATE          = "34000";
  }
  return psCda_def;

}  /* cursor() */


void TOci7xManager::updateStatus (const int iCODE) const
{

  if ( iCODE < 0 )
  {
    /*
    *  Condition:           connection exception
    *  Class subcondition:  transaction resolution unknown
    */
    SQLCODE = -1;
    SQLSTATE = "08007";
  }
  switch (iCODE)
  {
    case 0:
    {
      /*
      *  Condition:  successful completion
      */
      SQLCODE = 0; SQLSTATE = "00000";
      break;
    }
    /*
    *  ORA-00942:  table or view does not exist
    *  ORA-00955:  name is already used by an existing object
    */
    case -942:
    case -955:
    {
      /*
      *  Condition:  syntax error or access rule violation
      */
      SQLCODE = -1; SQLSTATE = "42000";
      break;
    }
    /*
    *  ORA-01012:  not logged on 
    */
    case -1012:
    {
      /*
      *  Condition:           connection exception
      *  Class subcondition:  connection does not exist
      */
      SQLCODE = -1; SQLSTATE = "08003";
      break;
    }
  }

}  /* updateStatus() */


