#include "postgresql-libpq-6.2.x.hh"


/*
*  UESQLC::postgresql-libpq-6.2.x template explicit instantiations.
*/

template class TCollection<string>;
template class TBaseContainer<string>;
template class TLinkedContainer<string>;
template class TCollection<PGconn*>;
template class TBaseContainer<PGconn*>;
template class TLinkedContainer<PGconn*>;
template class TBaseMap<string, PGconn*>;
template class TLinkedMap<string, PGconn*>;


/*
*  UESQLC::postgresql-libpq-6.2.x internal variables initiation.
*/

string                 TPostgresql62xManager::stSqlInstruction;
PGconn*                 TPostgresql62xManager::ptPgconn       = NULL;
PGresult*               TPostgresql62xManager::ptPgresult     = NULL;
PGresult*               TPostgresql62xManager::ptAuxPgresult  = NULL;
char*                   TPostgresql62xManager::pcHost         = NULL;
char*                   TPostgresql62xManager::pcDatabaseName = getenv ("USER");


/*
*  C O N S T R U C T O R S
*/

TPostgresql62xManager::~TPostgresql62xManager (void)
{

  finish();

}  /* ~TPostgresql62xManager() */


bool TPostgresql62xManager::initiate (void)
{

  bool   gSuccess = gIsThereConnection;

  if ( !gIsThereConnection )
  {
    ptPgconn = PQsetdb (NULL, NULL, NULL, NULL, getenv ("USER"));
    if ( PQstatus (ptPgconn) == CONNECTION_BAD )
    {
      /*
      *  Condition:           connection exception
      *  Class subcondition:  SQL-client unable to establish SQL-connection
      */
      SQLCODE  = -1;
      SQLSTATE = "08001";
    }
    else
    {
      SQLCODE  = 0;
      SQLSTATE = "00000";
      gSuccess = true;
      connect ("DEFAULT", ptPgconn);
    }
  }
  return gSuccess;

}  /* initiate() */


void TPostgresql62xManager::connect (const string& rkyCONNECTION_NAME, const PGconn* pktCONNECTION)
{

  tConnectionMap.bind (rkyCONNECTION_NAME, (PGconn*) pktCONNECTION);
  stCurrentConnection = rkyCONNECTION_NAME;
  gIsThereConnection  = true;

}  /* connect() */


void TPostgresql62xManager::disconnect (const string& rkyCONNECTION_NAME)
{

  PQfinish (tConnectionMap [rkyCONNECTION_NAME]);
  tConnectionMap.unbind (rkyCONNECTION_NAME);
  if ( rkyCONNECTION_NAME == stCurrentConnection )
  {
    stCurrentConnection.erase();
    gIsThereConnection = false;
  }

}  /* disconnect() */


void TPostgresql62xManager::disconnectCurrent (void)
{

  disconnect (stCurrentConnection);

}  /* disconnect() */


PGconn* TPostgresql62xManager::setCurrentConnection (const string& rkyCONNECTION_NAME)
{

  PGconn*   ptConnectionPgconn = NULL;

  if ( tConnectionMap.isBinded (rkyCONNECTION_NAME) )
  {
    stCurrentConnection = rkyCONNECTION_NAME;
    ptConnectionPgconn  = tConnectionMap [rkyCONNECTION_NAME];
  }
  return ptConnectionPgconn;

}  /* setCurrentConnection() */


/*
*  S E L E C T O R S
*/

bool TPostgresql62xManager::existsDefaultConnection (void) const
{

  return !stCurrentConnection.empty();

}  /* existsDefaultConnection() */


bool TPostgresql62xManager::isConnected (const string& rkyCONNECTION_NAME) const
{

  return tConnectionMap.isBinded (rkyCONNECTION_NAME);

}  /* isConnected() */


PGconn* TPostgresql62xManager::currentConnection (void) const
{

  PGconn*   ptCurrentPgconn = NULL;

  if ( !stCurrentConnection.empty() )
  {
    ptCurrentPgconn = tConnectionMap [stCurrentConnection];
  }
  return ptCurrentPgconn;

}  /* currentConnection() */


PGconn* TPostgresql62xManager::connection (const string& rkyCONNECTION_NAME) const
{

  return tConnectionMap [rkyCONNECTION_NAME];

}  /* connection() */


void TPostgresql62xManager::finish (void)
{

  for (register size_t J = 0, zMapCardinal = tConnectionMap.linksCardinal(); ( J < zMapCardinal ) ;J++)
  {
    PQfinish (tConnectionMap.item (J));
  }
  tConnectionMap.clear();
  stCurrentConnection.erase();

}  /* finish() */


