/*
*  Name:      test-base-1.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 1999/04/20 14:04:18 $
*  Revision:  $Revision: 1.1 $
*
*  Copyright (C) 1996-1998  Rafael Jesus Alcantara Perez <ralcan@vnet.es>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/string.hh>
#include <mpcl/config_processor.hh>

using namespace mpcl;


string GetErrorMessage (void)
{

  string   yMessage;

  if ( SQLSTATE == "00000" )
  {
    yMessage = "successful completion";
  }
  else if ( SQLSTATE == "02000" )
  {
    yMessage = "no data";
  }
  else if ( SQLSTATE == "08001" )
  {
    yMessage = "connection exception: SQL-client unable to establish SQL-connection";
  }
  else if ( SQLSTATE == "08002" )
  {
    yMessage = "connection exception: connection name in use";
  }
  else if ( SQLSTATE == "08003" )
  {
    yMessage = "connection exception: connection does not exist";
  }
  else if ( SQLSTATE == "08007" )
  {
    yMessage = "connection exception: transaction resolution unknown";
  }
  else if ( SQLSTATE == "21000" )
  {
    yMessage = "cardinality violation";
  }
  else if ( SQLSTATE == "34000" )
  {
    yMessage = "invalid cursor name";
  }
  else if ( SQLSTATE == "40003" )
  {
    yMessage = "transaction rollback: statement completion unknown";
  }
  else if ( SQLSTATE == "42000" )
  {
    yMessage = "syntax error or access rule violation";
  }
  else
  {
    yMessage = "unknown error";
  }
  return yMessage;

}  /* GetErrorMessage() */


/*
*  Parameters and arguments:
*
*    {--hostname, -o}=HOSTNAME
*    {--username, -u}=USERNAME
*    {--password, -p}=PASSWORD
*/
int main (int argc, const char* argv[])
{

  TConfigProcessor   tConfig;
  size_t             zRecordCount = 0;

  TEST_INIT ("test-base-1");
  TEST_START_WATCH;

  string   yVar0;
  string   yVar1;
  string   yVar2;
  string   yAttribName;
  string   yCursorName;
  string   yUserName;
  string   yHostName;
  string   yCodeValue = "'002'";

  /*
  *  Define string constants names for options.
  */ 
  tConfig.addOption ("hostname", "o", "localhost");
  tConfig.addOption ("username", "u", getenv ("USER"));
  tConfig.addOption ("password", "p", "");
  tConfig.setCommandLineParameters (argc, argv);
  tConfig.processOptions();
  
  yHostName = Format ("'%s'", tConfig ["hostname"].value().c_str());
  if ( tConfig ["password"].value().empty() )
  {
    yUserName = Format ("'%s'", tConfig ["username"].value().c_str());
  }
  else
  {
    yUserName = Format ( "'%s/%s'"                            ,
                         tConfig ["username"].value().c_str() ,
                         tConfig ["password"].value().c_str() );
  }

  TEST_WRITE_INFO ("HOSTNAME = %s", yHostName.c_str());
  TEST_WRITE_INFO ("USER     = %s", yUserName.c_str());
  if ( !yHostName.empty() && !yUserName.empty() )
  {
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;
      whenever not found goto not_found;
      
      --
      --  CONNECT TO
      --
      connect to :yHostName user :yUserName;

      --
      --  CREATE TABLE
      --
      create table JOBS
      (
        CODE         char(3) not null ,
        WORKER_COUNT integer
      );

      create table WORKERS
      (
        WORK_CODE char(3)  not null ,
        NAME      char(40) not null ,
        HOURS     decimal(5)
      );

      --
      --  ALTER TABLE
      --
      alter table WORKERS add column STATE char (40);
      alter table WORKERS add column PHONE_NUMBER integer;

      --
      --  DELETE
      --
      delete from JOBS where CODE = :yCodeValue;
      delete from JOBS where CODE = '001';
      delete from WORKERS;

      --
      --  INSERT
      --
      insert into JOBS values ('001', 100);
      insert into JOBS values ('002', 250);
      insert into WORKERS values ('001', 'myname',   40, 'Andalucia',   5550000);
      insert into WORKERS values ('001', 'yourname', 35, 'Extremadura', 5550001);
      insert into WORKERS values ('001', 'hisname',  25, 'Catalunya',   5550002);
      insert into WORKERS values ('002', 'john',     35, 'New England', 5550003);
      insert into WORKERS values ('002', 'james',    35, 'Washington',  5550004);
      insert into WORKERS select WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER from WORKERS where NAME = "myname";
      insert into WORKERS select WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER from WORKERS;

      --
      --  SELECT for a single row.
      --
      --    Cardinality violation, so ( SQLSTATE == '21000' ).
      --
      whenever sqlerror goto cardinality_check;
      select HOURS into :yVar1 from WORKERS, JOBS where WORK_CODE = CODE;
    }
    TEST_FAIL;


  cardinality_check:

    TEST_STRINGS (SQLSTATE.c_str(), "21000");
    yAttribName = "WORK_CODE";
    yCursorName = "MYCURSOR";
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;

      --
      --  SELECT for a single row.
      --
      select count (*) into :yVar2 from WORKERS where name = 'myname';

      --
      --  UPDATE
      --
      update WORKERS
      set PHONE_NUMBER = WORKERS.PHONE_NUMBER
      where WORKERS.NAME = 'myname' and WORKERS.NAME = NAME;

      --
      --  REVOKE
      --
      revoke all privileges on JOBS from public cascade;
      revoke all privileges on WORKERS from public restrict;

      --
      --  GRANT
      --
      grant select on JOBS to public;

      --
      --  CREATE VIEW
      --
      create view VIEW1 as select * from JOBS;

      --
      --  DROP VIEW
      --
      drop view VIEW1 cascade;

      --
      --  DECLARE CURSOR/OPEN/FETCH/CLOSE
      --
      declare MYCURSOR cursor for select CODE from JOBS where CODE = '001';
      open MYCURSOR;
      fetch from :yCursorName into :yVar0;
      close MYCURSOR;

      --
      --  DROP TABLE
      --
      drop table JOBS cascade;
    }

    TEST_STRINGS (SQLSTATE.c_str(), "00000");
    TEST_STRINGS (yVar2.c_str(),   "4");
    TEST_STRINGS (yVar0.c_str(),   "001");

    extern "UESQL::default"
    {
      --
      --  DECLARE CURSOR/OPEN/WHENEVER
      --
      declare MYCURSOR cursor for select :yAttribName, NAME, HOURS from WORKERS;
      open MYCURSOR;
      whenever not found goto loop_end;
    }
    
    if ( _ptTargetFile )
    {
      fprintf (_ptTargetFile, "%s\n", "WORK_CODE NAME                                     HOURS");
      fprintf (_ptTargetFile, "%s\n", "========= ======================================== =====");
    }
    for (;;)
    {
      extern "UESQL::default"
      {
        fetch from MYCURSOR into :yVar0, :yVar1, :yVar2;
      }
      if ( _ptTargetFile )
      {
        fprintf (_ptTargetFile, "%9s %40s %5s\n", yVar0.c_str(), yVar1.c_str(), yVar2.c_str());
      }
      ++zRecordCount;
    }


  loop_end:

    extern "UESQL::default"
    {
      --
      --  CLOSE CURSOR
      --
      close MYCURSOR;

      --
      --  DISCONNECT
      --
      disconnect current;
    }

    extern "UESQL::default"
    {
      --
      --  CONNECT TO
      --
      connect to :yHostName as 'myconnection' user :yUserName;

      --
      --  DROP TABLE
      --
      drop table WORKERS cascade;
    }
    goto end;

    
  error:

    cerr << "SQLSTATE="    << SQLSTATE << endl;
    cerr << "SQLCODE="     << SQLCODE  << endl;
    cerr << "DESCRIPTION=" << GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (SQLCODE);  
    goto end;

    
  not_found:

    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH;
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  /* main() */

